<?php
/*
 * Created on 9 Aug 2007 
 * By julien reichel <julien.reichel@spinetix.com>
 */

$title="Network Settings"; 
$tabs=array("Basic","Advanced");

include ("prolog.php");

include 'safe-data.php';

function getMAC(){
	$info=`ifconfig eth0`;
	$macp=strpos($info,"HWaddr ");
	if ($macp===false) return "";
	$macp=$macp+strlen("HWaddr ");
	$mac = substr($info, $macp, strpos($info," ",$macp)-$macp);
	return $mac;
}

class ipconfig{
	var $dhcp=1;
	var $address="";
	var $netmask="";
	var $gateway="";
	var $hostname="";

	var $startTag="# START-spxsysconfig DO NOT REMOVE THE START AND END TAGS";
	var $endTag="# END-spxsysconfig";
	var $fname="/etc/network/interfaces";
	function mask($adr,$msk){
		$a=ip2long($adr);
		$m=ip2long($msk);
		return long2ip($a & $m);
	}
	function broad($adr,$msk){
		$a=ip2long($adr);
		$m=ip2long($msk);
		
		return long2ip(($a & $m)+(~$m));
	}

	function load(){
		 // get the config
		$interface=file_get_contents($this->fname);
		if (!isset($interface) && $interface) return;
	
		$start=strpos($interface,$this->startTag);
		$end=strpos($interface,$this->endTag);
		if ($start===false || $end===false) return;
		
		
		$str1="iface eth0 inet ";
		$iface=strpos($interface,$str1, $start);
		$iface+=strlen($str1);
		$conf=substr($interface,$iface,$end-$iface);
		$vals=explode ("\n",$conf);	
		
		if (trim($vals[0])=="dhcp"){
			$this->dhcp=1;
			foreach ($vals as $name){
				if (!(($n=strpos($name,"hostname "))===false)){
					$this->hostname=trim(substr($name, strlen("hostname ")));
				}
			}
		}elseif (trim($vals[0])=="static"){
			$this->dhcp=0;
			foreach ($vals as $name){
				if (!(($n=strpos($name,"address "))===false)){
					$this->address=trim(substr($name, strlen("address")+$n));
				}elseif (!(($n=strpos($name,"netmask "))===false)){
					$this->netmask=trim(substr($name, strlen("netmask")+$n));
				}elseif (!(($n=strpos($name,"gateway "))===false)){
					$this->gateway=trim(substr($name, strlen("gateway")+$n));
				}
			}
		}
		
	}
	function save(){
		$conf="auto eth0\niface eth0 inet ";
		$hasError=false;
		$hasGateway=true;
		if (!$this->dhcp){
		     if (!ip_check($this->address)){ 
		     	echo "<p style='color:red'>Warning: '".htmlspecialchars($this->address)."' is not a valid address. Configuration not saved</p>"; 
		     	$hasError=true;
	     	} 
		     if (!netmask_check($this->netmask)){ 
		     	echo "<p style='color:red'>Warning: '".htmlspecialchars($this->netmask)."' is not a valid netmask. Configuration not saved</p>"; 
		     	$hasError=true;
	     	}
		     if ($this->gateway!=""){
		     	if (!ip_check($this->gateway)){ 
		     		echo "<p style='color:red'>Warning: ".htmlspecialchars($this->gateway)." is not a valid gateway. Configuration not saved</p>"; 
		     		$hasError=true;
		     	}
		     	if (!ipconf_check($this->address, $this->netmask, $this->gateway)){ 
		     		echo "<p style='color:red'>Warning: IP configuration is not valid. Configuration not saved</p>"; 
		     		$hasError=true;
		     	}
		     } else $hasGateway=false;
			
			if ($hasError) return false;
			
			$conf.="static\n";
			$conf.="\taddress $this->address\n";
			$conf.="\tnetwork ".$this->mask($this->address,$this->netmask)."\n";
			$conf.="\tnetmask $this->netmask\n";
			if ($hasGateway) $conf.="\tgateway $this->gateway\n";
			$conf.="\tbroadcast ".$this->broad($this->address,$this->netmask)."\n";
			$conf.="\tpre-up [ ! -x /etc/network/eth-inet-dad ] || /etc/network/eth-inet-dad\n";
			
		} else {
			$conf.="dhcp\n";
			if ($this->hostname!="")$conf.="\thostname $this->hostname\n";
		}
			

		$interface=file_get_contents($this->fname);
		if (!isset($interface) && $interface) return false;
	
		$start=strpos($interface,$this->startTag);
		$end=strpos($interface,$this->endTag);
		if ($start===false) $start=strlen($interface);	
		if ($end===false) {
			$end=$start;	
			$conf.=$this->endTag."\n";
		}
		if (!save_file($this->fname,substr_replace($interface,$this->startTag."\n".$conf,$start,$end-$start)))
			return false;

		// copy settings to failsafe-data
		if ($this->dhcp) {
		    $netconf = "dhcp\n";
		} else {
		    $netconf = "static=" . $this->address . ";" .
			$this->netmask . ";" .
			$this->broad($this->address,$this->netmask ) . ";";
		    if ($hasGateway)
			$netconf .= $this->gateway;
		    $netconf .= "\n";
		}

		if ( ! setSafeModeData("network", $netconf, FALSE) )
		    echo "<p style='color:red'>Failed saving network configuration for recovery console, please try again.</p>";

		return true;
	}
}
class dnsconfig{
	var $nameserver1="";
	var $nameserver2="";
	var $nameserver3="";
	var $search="";
	var $domain="";
	var $fname="/etc/network/resolv.conf.static";
	function load(){
		$dns=file_get_contents($this->fname);
		$tmp=$dns;
		$i=1;
		$pos=strpos($tmp,"nameserver");
		while (!($pos===false)){
			$pos+=strlen("nameserver");
			$name="nameserver".$i; 
			$i++;
			$this->$name=trim(substr($tmp,$pos,strpos($tmp,"\n",$pos)-$pos));
			$tmp=substr($tmp,strpos($tmp,"\n",$pos));
			$pos=strpos($tmp,"nameserver");
		}
			
		if (!($pos===false)) 
		$pos=strpos($dns,"search ");
		if (!($pos===false)) {$pos+=strlen("search "); $this->search=trim(substr($dns,$pos,strpos($dns,"\n",$pos)-$pos));}
		$pos=strpos($dns,"domain ");
		if (!($pos===false)) {$pos+=strlen("domain "); $this->domain=trim(substr($dns,$pos,strpos($dns,"\n",$pos)-$pos));}
	}
	function save(){
		$conf="";
		if ($this->nameserver1!="" ) 
			if (ip_check($this->nameserver1)) $conf.="nameserver $this->nameserver1\n"; 
			else { echo "<p style='color:red'>Warning: ".htmlspecialchars($this->nameserver1)." is not a valid DNS. Configuration not saved</p>"; return false;}
		if ($this->nameserver2!="" ) 
			if (ip_check($this->nameserver2)) $conf.="nameserver $this->nameserver2\n"; 
			else { echo "<p style='color:red'>Warning: ".htmlspecialchars($this->nameserver2)." is not a valid DNS. Configuration not saved</p>"; return false;}
		if ($this->nameserver3!="" ) 
			if (ip_check($this->nameserver3)) $conf.="nameserver $this->nameserver3\n"; 
			else { echo "<p style='color:red'>Warning: ".htmlspecialchars($this->nameserver3)." is not a valid DNS. Configuration not saved</p>"; return false;}
		
		if ($this->search!="") $conf.="search $this->search\n";
		if ($this->domain!="") $conf.="domain $this->domain\n";
		if ( $conf == "" ) $conf="\n"; 
		return save_file($this->fname,$conf);
	}
	function clear(){
	     $this->nameserver1="";
	     $this->nameserver2="";
	     $this->nameserver3="";
	     $this->search="";
	     $this->domain="";
	}
}

$conf=new ipconfig;
$conf->load();
$dns=new dnsconfig;
$dns->load();


//parse_str($_GET,$input);
$mod=0;
$moddns=0;

$reboot=0;
$query=0;
$reboot=false;
$reason = "";

if (isset($_POST["dhcp"])) {
	$vals=array("dhcp", "address", "netmask", "gateway", "hostname" );
	foreach ($vals as $name){
		if (!isset($_POST[$name])) $_POST[$name]=""; 
		if ($conf->$name!=$_POST[$name]) {
			$mod=1; 
			$conf->$name=$_POST[$name];	
		}
	}
	$vals=array("nameserver1","nameserver2","nameserver3", "search", "domain" );
	foreach ($vals as $name){
		if (!isset($_POST[$name])) $_POST[$name]=""; 
		if ($dns->$name!=$_POST[$name]) {
			$moddns=1; 
			$dns->$name=$_POST[$name];
		}
	}
	
	if (isset($_POST["reboot"])) {
		$reboot=true;
		$reason = "user forced reboot";
	}
	
	$allOK=true;
	if ($mod) $allOK=$conf->save();
	if ($moddns && $allOK) $allOK=$dns->save();
	
	if ( $allOK && ($mod || $moddns) ) {
		$reboot=true;
		$reason = "network config change (" . ($conf->dhcp ? "DHCP" : $conf->address) . ")";
	}
}

$snmpfile='/etc/snmp/snmpd.local.conf';
$snmpconf=file_get_contents($snmpfile);
$comunity="";

if (isset($_POST['rocommunity']) && isset($_POST['updateSNMP'])){
	$comunity=$_POST['rocommunity'];
	# NOTE: we set rocommunity and rocommunity6
	$snmpconf=preg_replace("@^(\s*rocommunity6?)\s+\S+.*@m","$1 $comunity default",$snmpconf);
	if ( save_file($snmpfile,$snmpconf) ) {
		$reboot=true; 	
		$reason = "snmp config change (" . $comunity .")";
	}
}

if (preg_match("@^\s*rocommunity\s+(\S+)@m",$snmpconf,$matches)){
	$comunity=$matches[1];
}

$xlinkNS = 'http://www.w3.org/1999/xlink';

$config_name="/srv/raperca/spxconfig.xml";
//$config_name="../config.xml";
$spxconf=file_get_contents($config_name);
$deviceName="";
$screenId="";
$remoteClientsEnabled="false";
$sharedVarPort=1234;
$proxyServer="";
$proxyPort="";
$proxyUser="";
$proxyPass="";
$bypassProxy=true;
$proxyFile = "/etc/network/spxproxy.conf";

if ($spxconf) {
	$domdoc = new DOMDocument;
	$domdoc->loadXML($spxconf);
	$multiscreen=$domdoc->getElementsByTagName("multiscreen");
	if ($multiscreen->length==1){
		$multiscreen=$multiscreen->item(0);
		if ($multiscreen->hasAttribute("screenId")) $screenId=$multiscreen->getAttribute("screenId");
	}else unset($multiscreen);
	$reporting=$domdoc->getElementsByTagName("reporting");
	if ($reporting->length==1){
		$reporting=$reporting->item(0);
		if ($reporting->hasAttribute("deviceName")) $deviceName=$reporting->getAttribute("deviceName");
	}else unset($reporting);
	$interactivity=$domdoc->getElementsByTagName("interactivity");
	if ($interactivity->length==1){
		$interactivity=$interactivity->item(0);
		if ($interactivity->hasAttribute("remoteClientsEnabled")) $remoteClientsEnabled=$interactivity->getAttribute("remoteClientsEnabled");
		if ($interactivity->hasAttribute("sharedVarPort")) $sharedVarPort=$interactivity->getAttribute("sharedVarPort");
	}else unset($interactivity);
	$proxyURI=$domdoc->getElementsByTagName("proxy");
	if ($proxyURI->length==1){
		$proxyURI=$proxyURI->item(0);
	}else unset($proxyURI);
	
	$proxydoc = new DOMDocument;
	if ( file_exists($proxyFile) ) {
		
		$proxydoc->loadXML(file_get_contents($proxyFile));
		$proxy=$proxydoc->getElementsByTagName('proxy');
		if ($proxy->length>0){
			$proxy=$proxy->item(0);
			if ($proxy->hasAttribute("server")) $proxyServer=$proxy->getAttribute("server");
			if ($proxy->hasAttribute("port")) $proxyPort=$proxy->getAttribute("port");	
			if ($proxy->hasAttribute("username")) $proxyUser=$proxy->getAttribute("username");
			if ($proxy->hasAttribute("password")) $proxyPass=$proxy->getAttribute("password");
			if ($proxy->hasAttribute("bypassLocalNetwork")) $bypassProxy=$proxy->getAttribute("bypassLocalNetwork")=="true";
			
			if ($proxyServer!="" && $proxyPort=="")
				$proxyPort="8080";
		} else unset($proxy);
	}
	
	
	if (isset($_POST['applyNames'])) {
		$deviceName = $_POST['deviceName'];
		$screenId = $_POST['screenId'];
		if ( preg_match('/[^\w\d\.]/', $deviceName) ) {
			echo "<p style='color:red'>Warning: Device name ('$deviceName') is not valid. The name may contain only ASCII letters, digits and the hyphen</p>";
			$deviceName="";
		}
		
		$player = $domdoc->getElementsByTagName("player");
		if ($player->length==0){
			$player = $domdoc->createElement("player");
			$domdoc->documentElement->appendChild($player);
		} else $player=$player->item(0);
		if (!isset($multiscreen)){
			$multiscreen = $domdoc->createElement("multiscreen");
			$player->appendChild($multiscreen);
		} 
		if (!isset($reporting)){
			$reporting = $domdoc->createElement("reporting");
			$player->appendChild($reporting);
		}
		
		if ($serial!=$deviceName)
			$reporting->setAttribute("deviceName",$deviceName);
		else
			$reporting->setAttribute("deviceName", "");		
		if ($serial!=$screenId)
			$multiscreen->setAttribute("screenId", $screenId);
		else
			$multiscreen->setAttribute("screenId", "");
		
		save_file($config_name,$domdoc->saveXML(), true);	
		reconfig_player();
	}	
	if (isset($_POST['updateAPI'])) {
		if (isset($_POST['remoteClientsEnabled'])) $remoteClientsEnabled="true";
		else $remoteClientsEnabled="false";
		if (isset($_POST['sharedVarPort']) && $_POST['sharedVarPort']!="") $sharedVarPort=$_POST['sharedVarPort'];
		$player = $domdoc->getElementsByTagName("player");
		if ($player->length==0){
			$player = $domdoc->createElement("player");
			$domdoc->documentElement->appendChild($player);
		} else $player=$player->item(0);
		if (!isset($interactivity)){
			$interactivity = $domdoc->createElement("interactivity");
			$player->appendChild($interactivity);
		} 
		$interactivity->setAttribute("remoteClientsEnabled", $remoteClientsEnabled);
		$interactivity->setAttribute("sharedVarPort", $sharedVarPort);
		
		save_file($config_name,$domdoc->saveXML(), true);
		reconfig_player();
	}
	if (isset($_POST['updateProxy'])) {
		if (isset($_POST['proxyServer'])) $proxyServer=$_POST['proxyServer'];
		else $proxyServer="";
		if (isset($_POST['proxyPort']) && $_POST['proxyPort']!="" ) $proxyPort=$_POST['proxyPort'];
		else $proxyPort="8080";
		if (isset($_POST['proxyUser']) && $_POST['proxyUser']!="" ) $proxyUser=$_POST['proxyUser'];
		if (isset($_POST['proxyPass']) && $_POST['proxyPass']!="" ) $proxyPass=$_POST['proxyPass'];
		if ( isset($_POST['bypassProxy']) )  $bypassProxy=true; else $bypassProxy=false;
		
		if ( $proxyServer=="" ) {
			$proxyPort="";
			$proxyUser="";
			$proxyPass="";	
		}
		
		$resource = $domdoc->getElementsByTagName("resource");
		if ($resource->length==0){
			$resource = $domdoc->createElement("resource");
			$domdoc->documentElement->appendChild($resource);
		} else $resource=$resource->item(0);
		if (!isset($proxyURI)){
			$proxyURI = $domdoc->createElement("proxy");
			$resource->appendChild($proxyURI);
		} 
		$proxyURI->setAttributeNS($xlinkNS,"xlink:href", $proxyFile);
		
		$proxyconf = $proxydoc->getElementsByTagName("proxyconf");
		if ($proxyconf->length==0){
			$proxyconf = $proxydoc->createElement("proxyconf");
			$proxydoc->appendChild($proxyconf);
		} else $proxyconf=$proxyconf->item(0);
		if ( $proxyServer!="" ) {
			if (!isset($proxy)){
				$proxy = $proxydoc->createElement("proxy");
				$proxyconf->appendChild($proxy);
			} 
			$proxy->setAttribute("server", $proxyServer);
			$proxy->setAttribute("port", $proxyPort);
			$proxy->setAttribute("bypassLocalNetwork", $bypassProxy?"true":"false");
			if ($proxyUser!="") {
				$proxy->setAttribute("username", $proxyUser);	
				$proxy->setAttribute("password", $proxyPass);
			}
		} else if ( isset($proxy) ) {
			$proxyconf->removeChild($proxy);
		}
		save_file($config_name,$domdoc->saveXML(), true);
		save_file($proxyFile,$proxydoc->saveXML(), false);
		$reboot=true;
		$reason = "proxy config change (" . ($proxyServer=="" ? "No Proxy" : $proxyServer) . ")";
	}
}
if ($deviceName=="") $deviceName=$serial;
if ($screenId=="") $screenId=$serial;

if ($remoteClientsEnabled=="true") $remoteClientsEnabled=true;
else $remoteClientsEnabled=false;
if ($reboot){

     shutdown($reason);

if ($conf->dhcp)print("<br/><h3>DHCP will be used</h3>");
else print("<br/><h3>New ip will be <a href='http://$conf->address/'>$conf->address</a></h3>");

}else{
if ($conf->dhcp) $dns->clear();
if ($tabIdx==1) { // normal  
?>
<script type="text/javascript">
<!--
function useDHCP(enabled)
{
	document.getElementsByName("address")[0].disabled=enabled;
	document.getElementsByName("netmask")[0].disabled=enabled;
	document.getElementsByName("gateway")[0].disabled=enabled;
	document.getElementsByName("nameserver1")[0].disabled=enabled;
	document.getElementsByName("nameserver2")[0].disabled=enabled;
	document.getElementsByName("nameserver3")[0].disabled=enabled;
	document.getElementsByName("domain")[0].disabled=enabled;
}
-->
</script>

<h2>Identification: </h2>
<form action="ipconfig.php" method="post">
<table>
<?php
       print("<tr><td>Device name: </td><td><input size='30' type='text' name='deviceName' value=\"".htmlspecialchars($deviceName)."\" /></td></tr>\n");
     print("<tr><td>Multiscreen ID: </td><td><input size='30' type='text' name='screenId' value=\"". htmlspecialchars($screenId). "\" /></td></tr>\n");
?>
</table>
<input type="submit" value="Apply" name="applyNames"/><br/><br/>

</form>

<h2>Network Settings: </h2>
<form action="ipconfig.php" method="post">

<table id='IpTable'>
<?php

print ("<tr><td>DHCP</td><td><input type='radio' name='dhcp' value='1' onclick='useDHCP(true)' ");
if ($conf->dhcp) print ("checked='checked' /> </td></tr>\n");
else print ("/></td></tr>\n");
//print("<tr><td/><td>Hostname:</td><td><input size='30' type='text' name='hostname' value=\"". htmlspecialchars($conf->hostname). "\"/></td></tr>");


print ("<tr><td>Static</td><td><input type='radio' name='dhcp' value='0' onclick='useDHCP(false)' ");
if (!$conf->dhcp) print ("checked='checked' /></td></tr>\n");
else print ("/></td></tr> \n");

if ($conf->dhcp) $disableConf="disabled='disabled' ";
else $disableConf=" ";
print("<tr><td/><td>Address:</td><td><input size='30' type='text' name='address' value=\"". htmlspecialchars($conf->address). "\" $disableConf/> e.g. 192.168.1.10</td></tr>\n");
print("<tr><td/><td>Netmask:</td><td><input size='30' type='text' name='netmask' value=\"". htmlspecialchars($conf->netmask). "\" $disableConf/> e.g. 255.255.255.0</td></tr>\n");
print("<tr><td/><td>Gateway:</td><td><input size='30' type='text' name='gateway' value=\"". htmlspecialchars($conf->gateway). "\" $disableConf/> e.g. 192.168.1.1</td></tr>\n");
print ("<tr><td/><td>DNS:</td></tr>\n");
print("<tr><td/><td>Server 1:</td><td><input size='30' type='text' name='nameserver1' value=\"". htmlspecialchars($dns->nameserver1). "\" $disableConf/> e.g. 192.168.1.1</td></tr>\n");
print("<tr><td/><td>Server 2:</td><td><input size='30' type='text' name='nameserver2' value=\"". htmlspecialchars($dns->nameserver2). "\" $disableConf/> </td></tr>\n");
print("<tr><td/><td>Server 3:</td><td><input size='30' type='text' name='nameserver3' value=\"". htmlspecialchars($dns->nameserver3). "\" $disableConf/> </td></tr>\n");
print("<tr><td/><td>DNS sufix:</td><td><input size='30' type='text' name='domain' value=\"". htmlspecialchars($dns->domain). "\" $disableConf/></td></tr>\n");
?>

</table>

<input type="submit" value="Apply" name="updateIP" /><br/><br/>
</form>
<?php
} else { // advanced settings
?>
<h2>Network API:</h2>
<br/>
<form action="ipconfig.php" method="post" >
<input type='checkbox' name='remoteClientsEnabled' <?php if ($remoteClientsEnabled) echo "checked='checked'"; ?> /> 
Enable API server using port : 
<input size='6' type='text' name='sharedVarPort' value="<?php echo htmlspecialchars($sharedVarPort); ?>"/> <br/><br/>
<input type="submit" value="Apply" name="updateAPI"/>
</form><br/>

<h2>Proxy Settings:</h2>
<br/>
<form action="ipconfig.php" method="post" >
<p>
Server: <input size='30' type='text' name='proxyServer' value="<?php echo htmlspecialchars($proxyServer); ?>"/> 
Port: <input size='5' type='text' name='proxyPort' value="<?php echo htmlspecialchars($proxyPort); ?>"/>
</p>
<p>
Username: <input size='15' type='text' name='proxyUser' value="<?php echo htmlspecialchars($proxyUser); ?>"/>
 Password: <input size='15' type='password' name='proxyPass' value="<?php echo htmlspecialchars($proxyPass); ?>"/>
</p>
<p>
<input type='checkbox' name='bypassProxy' <?php if ($bypassProxy) echo "checked='checked'"; ?> /> 
Bypass proxy server for local addresses: 
</p>
<br/>
<input type="submit" value="Apply" name="updateProxy"/>
</form><br/>

<h2>SNMP Settings:</h2>
<br/>
<form action="ipconfig.php" method="post" >
Read only community: <input size='30' type='text' name='rocommunity' value="<?php echo htmlspecialchars($comunity); ?>"/><br/><br/>
<input type="submit" value="Apply" name="updateSNMP"/>
</form>
 
<?php
} // advanced settings
} // reboot IP
include "final.php"; 
?>


